import { AfterViewInit, Component, Input, ViewChild } from '@angular/core';
import { Button, MessageDialog, Scheduler, SchedulerConfig } from '@bryntum/schedulerpro';
import { BryntumSchedulerProComponent } from '@bryntum/schedulerpro-angular';
import { BackendService } from 'src/app/libs/infrastructure/backend.service';
import { OperationPlan, OperationPlanEventDataField, OperationPlanShipDataField } from 'src/app/libs/models/operation-plan.model';

@Component({
  selector: 'app-bryntum-custom-scheduler',
  templateUrl: './bryntum-custom-scheduler.component.html',
  styleUrls: ['./bryntum-custom-scheduler.component.scss']
})
export class BryntumCustomSchedulerComponent implements AfterViewInit {
  @ViewChild(BryntumSchedulerProComponent, { static : true })
  schedulerComponent!: BryntumSchedulerProComponent;

  @Input() schedulerConfig!: Partial<SchedulerConfig>;
  @Input() resourceData!: OperationPlanShipDataField[];
  @Input() eventData!: OperationPlanEventDataField[];
  @Input() startDate!: Date;
  @Input() endDate!: Date;

  private scheduler!: Scheduler;
  private reloadButton!: Button;

  constructor(private service: BackendService) {}

  ngAfterViewInit(): void {
    this.scheduler = this.schedulerComponent.instance;

    const { reloadButton } = this.scheduler.widgetMap as { reloadButton: Button };
    this.reloadButton = reloadButton;
    reloadButton.on({ click : this.onDataReload, thisObj : this })
  }

  private onDataReload() {
    this.scheduler.mask({
      text: "Loading new data..."
    })

    let subscription = this.service.getEditedSchedulerData()
      .subscribe({
        next: async (response: OperationPlan) => {
          this.scheduler.unmask();
          const result = await MessageDialog.confirm({
            title        : 'Success',
            message      :
              `<div style="display:flex;flex-direction:column;">
              Data loaded from backend!
              <br>Load data into scheduler?</div>
              `,
            okButton     : 'Yes',
            cancelButton : 'No'
          });

          if (result === MessageDialog.okButton) {
            this.eventData = response.eventData;
            this.resourceData = response.resourceData;
          }

          subscription.unsubscribe();
        },
        error: async (error: any) => {
          this.scheduler.unmask();
          await MessageDialog.alert({
            title   : 'Reload failed!',
            message : error.error.detail
          });
        }
      });

  }
}
