import { Component, OnDestroy } from '@angular/core';
import { SchedulerConfig } from '@bryntum/schedulerpro';
import { Subscription } from 'rxjs';
import { BackendService } from 'src/app/libs/infrastructure/backend.service';
import { schedulerConfig } from './test-case.config';
import { OperationPlan, OperationPlanEventDataField, OperationPlanShipDataField } from 'src/app/libs/models/operation-plan.model';


@Component({
  selector: 'app-test-case',
  templateUrl: './test-case.component.html',
  styleUrls: ['./test-case.component.scss']
})
export class TestCaseComponent implements OnDestroy {
  dataLoaded = false;
  schedulerConfig: Partial<SchedulerConfig> = schedulerConfig;
  resourceData: OperationPlanShipDataField[] = [];
  eventData: OperationPlanEventDataField[] = [];
  startDate: Date = new Date();
  endDate: Date = new Date();

  subscriptions: Subscription = new Subscription();

  constructor(private service: BackendService) {
    this.subscriptions.add(this.service.getSchedulerData().subscribe(response => {
      this.addEventData(response);
    }));
  }

  private addEventData(data: OperationPlan) {
    this.resourceData = data.resourceData;
    this.eventData = data.eventData;
    this.startDate = data.startDate;
    this.endDate = data.endDate;

    this.dataLoaded = true;
  }

  ngOnDestroy(): void {
    this.subscriptions.unsubscribe();
  }
}
