import { formatNumber } from "@angular/common";
import { DateHelper, EventModel, ResourceModel, SchedulerConfig } from "@bryntum/schedulerpro";

// Class for resources (columns)
export class Ship extends ResourceModel { }

// Class for events
export class Journey extends EventModel {
  get resizeValidationListItems(): string[] {
    return ['<li>Can only be adjusted respecting the ships speed limits.</li>'];
  }

  get resizeValidationText(): string {
    return 'Can only be adjusted respecting the ships speed limits.';
  }
}

export const schedulerConfig: Partial<SchedulerConfig> = {
  allowOverlap      : false,
  createEventOnDblClick : false,

  features : {
    scheduleTooltip : false,
    stripe       : true,
    timeRanges   : true,

    eventTooltip : {
      template : (data : any) => {
        const journey = Object.assign(new Journey, data.eventRecord);
        journey.type = data.eventRecord.data.type;

        return `
          ${journey.name ? `<div class="b-sch-event-title">${journey.name}</div> <br>` : ''}
          ${journey.resizeValidationListItems ? `<br><br><div class="restriction-title"><b>Restrictions:</b></div>
            <ul class="restriction-list">
              ${journey.resizeValidationListItems ? `${journey.resizeValidationListItems}` : ''}
            </ul>` : ''}
          `;
        }
    },
    eventDragCreate : {
      disabled : true
    },
    eventEdit : {
      disabled : true
    }
  },

  subGridConfigs : {
    locked : { width : 150 }
  },

  columns : [
    {
      draggable : false,
      editor : undefined,
      flex : 1,
      type : 'resourceInfo',
      text : 'Ship',
      showEventCount: false,
      showImage: false,
      showMeta: (resourceRecord: any) => {
        return `${resourceRecord.data.eventCount} journeys`;
      }
    }
  ],

  barMargin  : 2,
  rowHeight  : 50,

  readOnly: true,
  viewPreset : {
    timeResolution : {
      unit      : 'hour',
      increment : 1
    },
    headers : [
      {
        unit       : 'month',
        dateFormat : 'MMM YYYY'
      },
      {
        unit       : 'week',
        dateFormat : 'WW'
      }
    ]
  },

  autoAdjustTimeAxis : false,
  multiEventSelect : true,

  zoomKeepsOriginalTimespan: true,
  minZoomLevel: 7,
  maxZoomLevel: 11,

  eventBodyTemplate : (data:any) => `${data.iconCls ? `<i class="${data.iconCls}"></i>` : ''}` +
    `<section>
      <div class="b-sch-event-header">${data.headerText}</div>
      <div class="b-sch-event-footer">${data.footerText}</div>
    </section>
  `,

  eventRenderer({ eventRecord, resourceRecord, renderData } : any) {
    return {
      headerText : eventRecord.type || '',
      footerText : eventRecord.name || '',
      iconCls    : eventRecord.iconCls
    };
  },

  tbar : [{
    type        : 'button',
    ref         : 'reloadButton',
    text        : 'Load Edited Data',
    toggleable  : false,
    disabled    : false
  }]
}
