import { DatePipe, DecimalPipe } from "@angular/common";
import { HttpClient } from "@angular/common/http";
import { Injectable } from "@angular/core";
import { OperationPlan, OperationPlanEventDataField, OperationPlanShipDataField } from "../models/operation-plan.model";
import { Observable, map } from "rxjs";
import { OperationPlanBackend, OperationPlanEventDataFieldBackend, OperationPlanShipDataFieldBackend } from "./operation-plan-backend.model";
import * as moment from "moment";

@Injectable({ providedIn: 'root' })
export class BackendService {
  private baseUrl = 'http://127.0.0.1:8000';

  constructor(private http: HttpClient, private datePipe: DatePipe, private decimalPipe: DecimalPipe) {}

  getSchedulerData(): Observable<OperationPlan> {
    return this.http.get<OperationPlanBackend>(this.baseUrl + `/v1/operation-plan`)
      .pipe(
        map(data => this.backendDataFromEntity(data))
      );
  }

  getEditedSchedulerData(): Observable<OperationPlan> {
    return this.http.get<OperationPlanBackend>(this.baseUrl + `/v1/operation-plan/edited`)
      .pipe(
        map(data => this.backendDataFromEntity(data))
      );
  }

  private backendDataFromEntity(backendData: OperationPlanBackend) : OperationPlan {
    const eventData: Array<OperationPlanEventDataField> = [];
    let currentResource: OperationPlanShipDataFieldBackend = backendData.resourceData[0];

    backendData.eventData.forEach(event => {
      if (currentResource.id != event.resourceId) {
        currentResource = backendData.resourceData.filter((resource) => resource.id == event.resourceId)[0];
      }
      eventData.push(
        this.eventDataFieldFromEntity(event, currentResource)
      );
    });

    const resourceData: Array<OperationPlanShipDataField> = [];
    backendData.resourceData.forEach(resource => {
      resourceData.push(
        this.resourceDataFieldFromEntity(resource)
      );
    });

    return {
      resourceData: resourceData,
      eventData : eventData,
      startDate: new Date(backendData.startDate),
      endDate: new Date(backendData.endDate),
    } as OperationPlan
  }

  private eventDataFieldFromEntity(event: OperationPlanEventDataFieldBackend, resource: OperationPlanShipDataFieldBackend) : OperationPlanEventDataField {
    return {
      id: event.id,
      resourceId: event.resourceId.toString(),
      name: event.name,
      startDate: moment.utc(resource.startDate).add(event.startTimeMilliseconds, 'millisecond').toDate(),
      endDate: moment.utc(resource.startDate).add(event.endTimeMilliseconds, 'millisecond').toDate(),
      type: event.type,
      eventColor: event.eventColor
    } as OperationPlanEventDataField
  }

  private resourceDataFieldFromEntity(resource: OperationPlanShipDataFieldBackend): OperationPlanShipDataField {
    return {
      id: resource.id.toString(),
      name: resource.name,
      startDate: new Date(resource.startDate),
      firstEventId: resource.firstEventId,
      eventCount: resource.eventCount
    } as OperationPlanShipDataField
  }

}
